package com.ibm.b2b.xmlmessage.framework;

import java.io.*;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
* A sample DOM writer. Adapted from sample program in xml4j package
*/
public class DOMWriter 
{
	protected PrintWriter out;

	public DOMWriter(Document document, String outfile)
	{
		try 
		{
			this.out = new PrintWriter(new BufferedWriter(new FileWriter(outfile)));
			print(document);
		}
		catch (Exception e)
		{
		}
	} 

	public DOMWriter(Document document, PrintWriter writer)
	{
		this.out = writer;
		print(document);
	} 

	/** Prints the specified node, recursively. */
	public void print(Node node) 
	{
		// is there anything to do?
		if ( node == null ) 
		{
			return;
		}

		int type = node.getNodeType();
		switch ( type ) 
		{
			// print document
			case Node.DOCUMENT_NODE: 
			{
				this.out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
				print(((Document)node).getDocumentElement());
				this.out.flush();
				break;
			}

			// print element with attributes
			case Node.ELEMENT_NODE: 
			{
				this.out.print('<');
				this.out.print(node.getNodeName());
				Attr attrs[] = sortAttributes(node.getAttributes());
				for ( int i = 0; i < attrs.length; i++ ) 
				{
					Attr attr = attrs[i];
					this.out.print(' ');
					this.out.print(attr.getNodeName());
					this.out.print("=\"");
					this.out.print(normalize(attr.getNodeValue()));
					this.out.print('"');
				}
				this.out.print('>');
				NodeList children = node.getChildNodes();
				if ( children != null ) 
				{
					int len = children.getLength();
					for ( int i = 0; i < len; i++ ) 
					{
						print(children.item(i));
					}
				}
				break;
			}

			// handle entity reference nodes
			case Node.ENTITY_REFERENCE_NODE: 
			{
				NodeList children = node.getChildNodes();
				if ( children != null ) 
				{
					int len = children.getLength();
					for ( int i = 0; i < len; i++ ) 
					{
						print(children.item(i));
					}
				}
				break;
			}

			// print cdata sections
			case Node.CDATA_SECTION_NODE: 
			{
				this.out.print(normalize(node.getNodeValue()));
				break;
			}

			// print text
			case Node.TEXT_NODE: 
			{
				this.out.print(normalize(node.getNodeValue()));
				break;
			}

			// print processing instruction
			case Node.PROCESSING_INSTRUCTION_NODE: 
			{
				this.out.print("<?");
				this.out.print(node.getNodeName());
				String data = node.getNodeValue();
				if ( data != null && data.length() > 0 ) 
				{
					this.out.print(' ');
					this.out.print(data);
				}
				this.out.print("?>");
				break;
			}
		}

		if ( type == Node.ELEMENT_NODE ) 
		{
			this.out.print("</");
			this.out.print(node.getNodeName());
			this.out.print('>');
		}

		this.out.flush();

	} // print(Node)

	/** 
	* Returns a sorted list of attributes. 
	*/
	protected Attr[] sortAttributes(NamedNodeMap attrs) 
	{
		int len = (attrs != null) ? attrs.getLength() : 0;
		Attr array[] = new Attr[len];
		for ( int i = 0; i < len; i++ )
		{
			array[i] = (Attr)attrs.item(i);
		}
			
		for ( int i = 0; i < len - 1; i++ )
		{
			String name  = array[i].getNodeName();
			int    index = i;
			for ( int j = i + 1; j < len; j++ )
			{
				String curName = array[j].getNodeName();
					
				if ( curName.compareTo(name) < 0 )
				{
					name  = curName;
					index = j;
				}
			}
				
			if ( index != i )
			{
				Attr temp    = array[i];
				array[i]     = array[index];
				array[index] = temp;
			}
		}
		return (array);
	} // sortAttributes(NamedNodeMap):Attr[]


	/** Normalizes the given string. */
	protected String normalize(String s)
	{
		StringBuffer str = new StringBuffer();

		int len = (s != null) ? s.length() : 0;

		for ( int i = 0; i < len; i++ )
		{
			char ch = s.charAt(i);
			switch ( ch )
			{
				case '<':
				{
					str.append("&lt;");
					break;
				}
				case '>':
				{
					str.append("&gt;");
					break;
				}
				case '&':
				{
					str.append("&amp;");
					break;
				}
				case '"':
				{
					str.append("&quot;");
					break;
				}
				case '\r':
				case '\n':
				{
				}
				default:
				{
					str.append(ch);
				}
			}
		}

		return (str.toString());
	} // normalize(String):String
} 
