package com.ibm.b2b.xmlmessage.framework;

import java.io.StringReader;

import org.apache.xerces.framework.XMLParser;
import org.apache.xerces.parsers.DOMParser;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXParseException;
import org.xml.sax.*;
import org.w3c.dom.*;

public class Factory implements java.io.Serializable
{
	String			packageName;
	String			documentRoot;
	String			dtdName;
	Document		document;
	DOMParser		parser;
	boolean			validation;
	
	public Factory()
	{
		this.document		= null;
		this.parser			= new DOMParser();
		this.validation		= false;
		this.documentRoot	= null; //"file:E:\\Develoment\\classpath\\";
		this.dtdName		= null; //"fc.xsd";
	}

	/**
	* Set the name of the Java package
	*/
	public void setPackageName(String packageName)
	{
		this.packageName = packageName;
	}

	/**
	* Get the name of the Java package
	*/
	public String getPackageName()
	{
		return this.packageName;
	}

	public void setErrorHandler(ErrorHandler handler)
	{
		this.parser.setErrorHandler(handler);
	}

	public void setValidation(boolean validation)
	{
		this.validation = validation;
	}

	public void setDocumentRoot(String documentRoot)
	{
		this.documentRoot = documentRoot;
	}

	/**
	* Set the name of the XML instance document
	*/
	/*  public void setXMLFilename(String filename)
	{
		this.xmlFile = filename;
	}
	*/

	/**
	* Get the name of the XML instance document
	*/
	/*  public String getXMLFilename()
	{
		return xmlFile;
	}
	*/

	/**
	* Load an existing instance document
	*/
	public IBMXMLElement loadDocument(String inBuff) throws SAXException, Exception
	{
/*
		if (this.documentRoot != null)
		{
			int				dtdStart	= inBuff.indexOf(this.dtdName);
			StringBuffer	buffer		= new StringBuffer(inBuff);
			buffer.insert(dtdStart, this.documentRoot);
			inBuff = buffer.toString();
		}
*/

		StringReader		reader	= new StringReader(inBuff);
		InputSource			source	= new InputSource(reader);

		try
		{
			//this.parser.setFeature("http://xml.org/sax/features/validation", this.validation);
			this.parser.parse(source);
			this.document				= this.parser.getDocument();
			Element			rootElement	= this.document.getDocumentElement();
			IBMXMLElement	domElement	= newInstance(rootElement);

			return domElement;
		}
		catch (SAXException saxe)
		{
			throw saxe;
		}
		catch (Exception e)
		{
			throw e;
			//System.out.println("*** LoadDocument ***" + e);
			//return null;
		}
	}

	/**
	* Save the DOM to the given print writer
	*/
	public void save(java.io.PrintWriter writer)
	{
		new DOMWriter(document, writer);
	}

	/**
	* Save the DOM to the specified file name
	*/
	public void save(String filename)
	{
		new DOMWriter(document, filename);
	}

	IBMXMLElement newInstance(Element rootElement)
	{
		String className = rootElement.getNodeName();

		try
		{
			String fullname = className;
			if (packageName != null)
			{
				fullname = packageName + "." + className;
			}

			Class javaClass = Class.forName(fullname);
			IBMXMLElement element = (IBMXMLElement) javaClass.newInstance();
			element.setElement(rootElement);
			element.setFactory(this);
			return element;
		}
		catch (Exception e)
		{
			System.out.println("*** Newinstance error ***" + e);
			return null;
		}
	}

	/**
	* Create a new DOM
	*   @param rootElementName - the root element of the DOM
	*/
	public IBMXMLElement createDOM(String rootElementName)
	{
		this.document = new org.apache.xerces.dom.DocumentImpl();
		return createElement(rootElementName, document);
	}

	/**
	* Create a new element in this document, and add it to the parentElement
	*/
	IBMXMLElement createElement(String name, Node parentElement)
	{
		Element childElement = this.document.createElement(name);
		parentElement.appendChild(childElement);

		return newInstance(childElement);
	}

	/**
	* Create a new attribute and add it to the parenetElement
	*/
	Attr createAttribute(String name, Element parentElement)
	{
		Attr attrNode = document.createAttribute(name);
		parentElement.setAttributeNode(attrNode);
		return attrNode;
	}
}
