package com.ibm.b2b.xmlmessage.framework;

import java.util.ArrayList;
import java.util.Iterator;

import org.w3c.dom.*;
//import com.ibm.xml.parsers.*;

/**
* Base class for all generated classes. Provides convenience
* methods to wrap the DOM
*/
public class IBMXMLElement implements java.io.Serializable
{
	private Element xmlElement;
	Factory factory;

	/**
	* Remember the XML element that corresponds to this element
	*/
	void setElement(Element element)
	{
		xmlElement = element;
	}

	/**
	* Remember the factory that creates this element
	*/
	void setFactory(Factory factory)
	{
		this.factory = factory;
	}

	/**
	* Find an element with the specified name
	*/
	public IBMXMLElement findElement(String name)
	{
		NodeList nodes = xmlElement.getElementsByTagName(name);
		int len = (nodes != null) ? nodes.getLength() : 0;

		for (int i=0; i < len; i++)
		{
			Node node = nodes.item(i);
			String item = node.getNodeName().trim();

			if (item.equals(name))
			{
				return factory.newInstance((Element) node);
			}
		}
		// return createElement(name);
		return null;
	}

	/**
	* Find an element with the specifed name and at the specified
	* location. If no element exists at that location this method
	* will create a new element.
	*
	*/
	/* @deprecated As of wobbet. This method has been replaced by
	* {@link getElementTypeAt} which does not create an element
	* if none is found but instead returns <code>null</code>.
	*/
	public IBMXMLElement findElement(String name, int index)
	{
		IBMXMLElement element = getElementTypeAt(name, index);
		/* if (element == null)
			element = createElement(name); */
					
		return element;
	}
	
	/**
	* Retrieves the element of the given name at the given
	* index. If no element of the given name exists at that
	* index then <code>null</code> is returned.
	*/
	public IBMXMLElement getElementTypeAt(String name, int index)
	{
		NodeList	nodes	= xmlElement.getElementsByTagName(name);
		int			len		= (nodes != null) ? nodes.getLength() : 0;

		for (int i=0; i < len; i++)
		{
			Node	node = nodes.item(i);
			String	item = node.getNodeName().trim();

			if (item.equals(name) && i == index)
				return factory.newInstance((Element)node);
		}
				
		return null;
	}
	
	/**
	* Returns an iterator to the list of sub-elements of the
	* given type. This iterator is a snapshot and does NOT
	* reflect changes to the DOM that occur after the call
	* to this method.
	*/
	public Iterator getElementsOfType(String type)
	{
		NodeList	nodes	= xmlElement.getElementsByTagName(type);
		final int	len		= (nodes != null) ? nodes.getLength() : 0;
		ArrayList	list	= new ArrayList(len);
		
		for (int i = 0; i < len; i++)
			{ list.add(nodes.item(i)); }
		
		return list.iterator();
	}

	/**
	* Create a new element and add it as a child to this node
	*/
	public IBMXMLElement createElement(String name)
	{
		// Verify that this element is valid
		return factory.createElement(name, xmlElement);
	}

	/**
	* Return the number of children with this name
	*/
	public int sizeOfElement(String name)
	{
		NodeList nodes = xmlElement.getElementsByTagName(name);
		int len = (nodes != null) ? nodes.getLength() : 0;

		return len;
	}

	/**
	* Return the node name for this element
	*/
	public String getElementName()
	{
		return xmlElement.getNodeName();
	}

	/**
	* Set the text value for this element
	* If created from new, TXElement is returned, and setNodeValue does
	* not work on TXElement.
	*/
	public void updateElementValue(String value)
	{
/*
		if (xmlElement instanceof TXElement)
		{
			((TXElement) xmlElement).addTextElement(new TXText(value));
		}
		else
*/
		{
			try
			{
				// try to update the node value
				xmlElement.setNodeValue(value);
			}
			catch (DOMException ex)
			{
				// fails, update PCDATA
				NodeList nList = xmlElement.getChildNodes();
				int len = (nList != null) ? nList.getLength() : 0;
				if (len > 0)
				{
					for(int i=0;i<len;i++)
					{
						if (nList.item(i) instanceof Text)
						{
							((Text)nList.item(i)).setData(value);
							return;
						}
					}
				}
				else
				{
					// if there isn't any, create a new child node of Text
					xmlElement.appendChild(factory.document.createTextNode(value));
				}
			}
		}
	}

	/**
	* Get the text value for this element
	*/
	public String getElementValue()
	{
/*
		if (xmlElement instanceof TXElement)
		{
			return ((TXElement) xmlElement).getText();
		}
		else
*/
		{
			// return node value
			String value = xmlElement.getNodeValue();
			if (value!=null)
			{
				return value;
			}

			// fails, return PCDATA
			NodeList nList = xmlElement.getChildNodes();
			for(int i=0;i<nList.getLength();i++)
			{
				if (nList.item(i) instanceof Text)
					return ((Text)nList.item(i)).getData();
			}
			return null;
		}
	}

	/**
	* Remove an element from the parent
	*/
	public boolean removeElement(IBMXMLElement element)
	{
		String name = element.xmlElement.getNodeName();
		NodeList nodes = xmlElement.getElementsByTagName(name);
		int len = (nodes != null) ? nodes.getLength() : 0;

		for (int i=0; i < len; i++)
		{
			Node node = nodes.item(i);
			String item = node.getNodeName().trim();

			if (item.equals(name))
			{
				try
				{
					xmlElement.removeChild(node);
					return true;
				}
				catch (Exception ex)
				{
					return false;
				}
			}
		}
		return false;
	}

	/**
	* Create a new attribute and add it to this element
	*/
	public Attr addAttribute(String name, String value)
	{
		Attr attrNode = factory.createAttribute(name, xmlElement);
		attrNode.setValue(value);
		return attrNode;
	}

	/**
	* Get the attribute value for the specified attribute name
	*/
	public String getAttributeValue(String attrname)
	{
		Attr attr = xmlElement.getAttributeNode(attrname);
		if (attr != null)
		{
			return attr.getValue();
		}
		return null;
	}

	/**
	* Set the attribute value for the specified attribute name
	*/
	public void setAttributeValue(String attrname, String attrvalue)
	{
		Attr attr = xmlElement.getAttributeNode(attrname);
		if (attr == null)
		{
			attr = addAttribute(attrname, attrvalue);
		}
		else
		{
			attr.setValue(attrvalue);
		}
	}

	/**
	* Remove the attribute from this element
	*/
	public boolean removeAttribute(String attrname)
	{
		try
		{
			xmlElement.removeAttribute(attrname);
			return true;
		}
		catch (Exception ex)
		{
			return false;
		}
	}

	public void createEmptyBody()
	{
	}
}
