#!perl
use XML::Parser;

@types;
my $schema      = $ARGV[0];
my $outputPath  = $ARGV[1];
my $packageName = $ARGV[2];
my $parser      = new XML::Parser(ErrorContext => 2);

$parser->setHandlers
	(
	Start	=> \&startHandler,
	End		=> \&endHandler,
	Char	=> \&charHandler
	);

$parser->parsefile($schema);

1;

sub pushType
{
    my ($type) = @_;
    push @types, $type;
    #print "pushing type $type\n";
}

sub pushElement
{
    if (scalar @types)
    {
        my ($element) = @_;
        @types[-1] = @types[-1] . "\n$element";
        #print "pushing element $element\n";
    }
}

sub addAttr
{
    my ($attr) = @_;
    @types[-1] = @types[-1] . "\n$attr";
}

sub popType
{
    my $type = pop @types;
    my @typeInfo = split /\n/, $type;
    
    foreach (@typeInfo)
    {
        if (my ($elName, $min, $max) = $_ =~ /(\w+)\t(\d)\t(\d|unbounded)/)
        {
            #print "elInfo name = $elName, min = $min, max = $max\n";
            if ($max eq "unbounded")
            {
                print FILE "public int get$elName", "Count()\n{\n";
                print FILE "return sizeOfElement(\"$elName\");\n}\n\n";
                print FILE "public $elName get$elName(int index)\n{\n";
                print FILE "return ($elName) findElement(\"$elName\", index);\n}\n\n";
            }
            else
            {
                print FILE "public $elName get$elName()\n{\n";
                print FILE "return ($elName) findElement(\"$elName\");\n}\n\n";
                #print FILE "\n";
                #print FILE "\n";
            }
        }
        elsif (my ($attrName, $type, $use) = $_ =~ /(\w+)\t(.+?)\t(optional|required)/)
        {
            #print "attrInfo name = $attrName, type = $type, use = $use\n";
            print FILE "public String get$attrName()\n{\n";
            print FILE "return getAttributeValue(\"$attrName\");\n}\n\n";
            print FILE "public void set$attrName(String attr)\n{\n";
            print FILE "setAttributeValue(\"$attrName\", attr);\n}\n\n";
        }
        else
        {
            s/(\w+)Type/$1/;
            #print "\ntypeName = $_\n";
            open(FILE, ">" . $outputPath . "\\" . $_ . ".java");
            print FILE "package $packageName;\n\n";
            print FILE "import com.ibm.b2b.xmlmessage.framework.*;\n\n";
            print FILE "public class $_ extends IBMXMLElement\n{\n";
            print FILE "public $_()\n{\n}\n\n";
        }
    }
    
    print FILE "}\n";
    close FILE;
}

sub popElement
{
    #print "popping element\n";
}

sub startHandler
{
	my $type	= @_[1];

	if ($type eq "xsd:complexType")
	{
	    my $elName = @_[3];
		pushType($elName);
	}
	elsif ($type eq "xsd:element")
	{
	    my $elName = @_[3];
	    my $elType = @_[5];
	    my $minOcc = @_[7];
	    my $maxOcc = @_[9];
	    
	    $minOcc=1,$maxOcc=1,unless defined $minOcc;
	    
	    pushElement("$elName\t$minOcc\t$maxOcc");
	}
	elsif ($type eq "xsd:attribute")
	{
		my $attrName = @_[3];
		my $attrType = @_[5];
		my $attrUse  = @_[7];
		$attrUse = "optional" unless defined $attrUse;
		addAttr("$attrName\t$attrType\t$attrUse");
	}
	else
	{
#		print "type = $type\n";
	}
}

sub endHandler
{
	my $type	= @_[1];

	if ($type eq "xsd:complexType")
	{
		popType();
	}
	elsif ($type eq "xsd:element")
	{
	    popElement();
	}
}

sub charHandler
{
}

